<?php

namespace GabrielGciaFdez\Validator\Constraint;

use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

class DateTimeRangeValidator extends ConstraintValidator
{
    const REGEX = '/^(\d{4})-(\d{2})-(\d{2}) (0[0-9]|1[0-9]|2[0-3]):([0-5][0-9]):([0-5][0-9])$/';

    /**
     * {@inheritDoc}
     */
    public function validate($value, Constraint $constraint)
    {
        if (null === $value) {
            return;
        }

        if ($value instanceof \DateTime) {
            $dateTimeValue = $value;
        } else if (preg_match(static::REGEX, $value, $matches) && checkdate($matches[2], $matches[3], $matches[1])) {
            $dateTimeValue = new \DateTime($value);
        } else {
            $this->context->addViolation($constraint->invalidMessage, array(
                '{{ value }}' => $value,
            ));

            return;
        }

        if (null !== $constraint->max && $dateTimeValue > $constraint->max) {
            $this->context->addViolation($constraint->maxMessage, array(
                '{{ value }}' => $this->formatDate($dateTimeValue),
                '{{ limit }}' => $this->formatDate($constraint->max),
            ));
        }

        if (null !== $constraint->min && $dateTimeValue < $constraint->min) {
            $this->context->addViolation($constraint->minMessage, array(
                '{{ value }}' => $this->formatDate($dateTimeValue),
                '{{ limit }}' => $this->formatDate($constraint->min),
            ));
        }
    }

    protected function formatDate($date)
    {
        return $date->format(\DateTime::W3C);
    }
}